@@ -1,5 +1,5 @@
 /*
-    Copyright 2016-2024 melonDS team
+    Copyright 2016-2025 melonDS team
 
     This file is part of melonDS.
 
@@ -339,9 +339,7 @@ const uint ResultAttrStart = ResultDepthStart+ScreenWidth*ScreenHeight*2;
 
 const char* Common = R"(
 
-#define TileSize 8
 const int CoarseTileCountX = 8;
-const int CoarseTileCountY = 4;
 const int CoarseTileW = (CoarseTileCountX * TileSize);
 const int CoarseTileH = (CoarseTileCountY * TileSize);
 
@@ -848,7 +846,7 @@ void main()
 
 const std::string ClearCoarseBinMask =
     BinningBuffer + R"(
-layout (local_size_x = 32) in;
+layout (local_size_x = ClearCoarseBinMaskLocalSize) in;
 
 void main()
 {
@@ -864,7 +862,7 @@ const std::string BinCombined =
     XSpanSetupBuffer +
     WorkDescBuffer + R"(
 
-layout (local_size_x = 32) in;
+layout (local_size_x = CoarseTileArea) in;
 
 bool BinPolygon(Polygon polygon, ivec2 topLeft, ivec2 botRight)
 {
