@@ -153,7 +153,9 @@ void EmuThread::run()
 
     while (emuStatus != emuStatus_Exit)
     {
-        MPInterface::Get().Process();
+        if (emuInstance->instanceID == 0)
+            MPInterface::Get().Process();
+
         emuInstance->inputProcess();
 
         if (emuInstance->hotkeyPressed(HK_FrameLimitToggle)) emit windowLimitFPSChange();
@@ -265,9 +267,6 @@ void EmuThread::run()
                 emuInstance->osdAddMessage(0, lid ? "Lid closed" : "Lid opened");
             }
 
-            // microphone input
-            emuInstance->micProcess();
-
             // auto screen layout
             {
                 mainScreenPos[2] = mainScreenPos[1];
