@@ -1,5 +1,5 @@
 /*
-    Copyright 2016-2024 melonDS team
+    Copyright 2016-2025 melonDS team
 
     This file is part of melonDS.
 
@@ -23,11 +23,7 @@
 
 #include <QPaintEvent>
 #include <QPainter>
-#ifndef _WIN32
-#ifndef APPLE
-#include <qpa/qplatformnativeinterface.h>
-#endif
-#endif
+
 #include <QDateTime>
 
 #include "OpenGLSupport.h"
@@ -50,6 +46,14 @@
 
 using namespace melonDS;
 
+#if !defined(_WIN32) && !defined(APPLE)
+#if QT_VERSION >= QT_VERSION_CHECK(6, 5, 0)
+using namespace QNativeInterface;
+#else
+#include <qpa/qplatformnativeinterface.h>
+#endif
+#endif
+
 
 const u32 kOSDMargin = 6;
 const int kLogoWidth = 192;
@@ -876,7 +880,7 @@ bool ScreenPanelGL::createContext()
     if (ourwin->getWindowID() != 0)
     {
         if (windowinfo.has_value())
-            if (glContext = parentwin->getOGLContext()->CreateSharedContext(*windowinfo))
+            if ((glContext = parentwin->getOGLContext()->CreateSharedContext(*windowinfo)))
                 glContext->DoneCurrent();
     }
     else
@@ -885,7 +889,7 @@ bool ScreenPanelGL::createContext()
                 GL::Context::Version{GL::Context::Profile::Core, 4, 3},
                 GL::Context::Version{GL::Context::Profile::Core, 3, 2}};
         if (windowinfo.has_value())
-            if (glContext = GL::Context::Create(*windowinfo, versionsToTry))
+            if ((glContext = GL::Context::Create(*windowinfo, versionsToTry)))
                 glContext->DoneCurrent();
     }
 
@@ -1020,6 +1024,8 @@ void ScreenPanelGL::deinitOpenGL()
     if (!glContext) return;
     if (!glInited) return;
 
+    glContext->MakeCurrent();
+
     glDeleteTextures(1, &screenTexture);
 
     glDeleteVertexArrays(1, &screenVertexArray);
@@ -1055,6 +1061,13 @@ void ScreenPanelGL::makeCurrentGL()
     glContext->MakeCurrent();
 }
 
+void ScreenPanelGL::releaseGL()
+{
+    if (!glContext) return;
+
+    glContext->DoneCurrent();
+}
+
 void ScreenPanelGL::osdRenderItem(OSDItem* item)
 {
     ScreenPanel::osdRenderItem(item);
@@ -1276,8 +1289,25 @@ std::optional<WindowInfo> ScreenPanelGL::getWindowInfo()
     wi.type = WindowInfo::Type::MacOS;
     wi.window_handle = reinterpret_cast<void*>(winId());
     #else
-    QPlatformNativeInterface* pni = QGuiApplication::platformNativeInterface();
     const QString platform_name = QGuiApplication::platformName();
+
+    #if QT_VERSION >= QT_VERSION_CHECK(6, 5, 0)
+    if (platform_name == QStringLiteral("xcb"))
+    {
+        wi.type = WindowInfo::Type::X11;
+        const QX11Application* x11 = qApp->nativeInterface<QX11Application>();
+        wi.display_connection = x11->display();
+        wi.window_handle = reinterpret_cast<void*>(winId());
+    }
+    else if (platform_name == QStringLiteral("wayland"))
+    {
+        wi.type = WindowInfo::Type::Wayland;
+        const QWaylandApplication* wl = qApp->nativeInterface<QWaylandApplication>();
+        wi.display_connection = wl->display();
+        wi.window_handle = reinterpret_cast<void*>(winId());
+    }
+    #else
+    QPlatformNativeInterface* pni = QGuiApplication::platformNativeInterface();
     if (platform_name == QStringLiteral("xcb"))
     {
         wi.type = WindowInfo::Type::X11;
@@ -1294,9 +1324,9 @@ std::optional<WindowInfo> ScreenPanelGL::getWindowInfo()
         wi.display_connection = pni->nativeResourceForWindow("display", handle);
         wi.window_handle = pni->nativeResourceForWindow("surface", handle);
     }
+    #endif
     else
     {
-        //qCritical() << "Unknown PNI platform " << platform_name;
         Platform::Log(Platform::LogLevel::Error, "Unknown PNI platform %s\n", platform_name.toStdString().c_str());
         return std::nullopt;
     }
